@@ -33,7 +33,9 @@
 
 #include "gvc-channel-bar.h"
 #include "gvc-balance-bar.h"
+#include "gvc-combo-box.h"
 #include "gvc-mixer-control.h"
+#include "gvc-mixer-card.h"
 #include "gvc-mixer-sink.h"
 #include "gvc-mixer-source.h"
 #include "gvc-mixer-source-output.h"
@@ -56,6 +58,10 @@ struct GvcMixerDialogPrivate
         GtkWidget       *effects_bar;
         GtkWidget       *output_stream_box;
         GtkWidget       *sound_effects_box;
+        GtkWidget       *hw_box;
+        GtkWidget       *hw_treeview;
+        GtkWidget       *hw_settings_box;
+        GtkWidget       *hw_profile_combo;
         GtkWidget       *input_box;
         GtkWidget       *output_box;
         GtkWidget       *applications_box;
@@ -65,7 +71,10 @@ struct GvcMixerDialogPrivate
         GtkWidget       *output_balance_bar;
         GtkWidget       *output_fade_bar;
         GtkWidget       *output_lfe_bar;
+        GtkWidget       *output_port_combo;
         GtkWidget       *input_treeview;
+        GtkWidget       *input_port_combo;
+        GtkWidget       *input_settings_box;
         GtkWidget       *sound_theme_chooser;
         GtkWidget       *click_feedback_button;
         GtkWidget       *audible_bell_button;
@@ -85,6 +94,17 @@ enum {
         NUM_COLUMNS
 };
 
+enum {
+        HW_ID_COLUMN,
+        HW_ICON_COLUMN,
+        HW_NAME_COLUMN,
+        HW_STATUS_COLUMN,
+        HW_PROFILE_COLUMN,
+        HW_PROFILE_HUMAN_COLUMN,
+        HW_SENSITIVE_COLUMN,
+        HW_NUM_COLUMNS
+};
+
 enum
 {
         PROP_0,
@@ -141,12 +161,30 @@ update_default_input (GvcMixerDialog *dialog)
         } while (gtk_tree_model_iter_next (model, &iter));
 }
 
+static void
+port_selection_changed (GvcComboBox *combo_box,
+                        const char  *port,
+                        GvcMixerDialog *dialog)
+{
+        GvcMixerStream *stream;
+
+        stream = g_object_get_data (G_OBJECT (combo_box), "stream");
+        if (stream == NULL) {
+                g_warning ("Could not find stream for port combo box");
+                return;
+        }
+        if (gvc_mixer_stream_change_port (stream, port) == FALSE) {
+                g_warning ("Could not change port for stream");
+        }
+}
+
 static void
 update_output_settings (GvcMixerDialog *dialog)
 {
         GvcMixerStream *stream;
         GvcChannelMap  *map;
         guint           num_settings;
+        const GList    *ports;
 
         g_debug ("Updating output settings");
         if (dialog->priv->output_balance_bar != NULL) {
@@ -164,6 +202,11 @@ update_output_settings (GvcMixerDialog *dialog)
                                       dialog->priv->output_lfe_bar);
                 dialog->priv->output_lfe_bar = NULL;
         }
+        if (dialog->priv->output_port_combo != NULL) {
+                gtk_container_remove (GTK_CONTAINER (dialog->priv->output_settings_box),
+                                      dialog->priv->output_port_combo);
+                dialog->priv->output_port_combo = NULL;
+        }
 
         stream = gvc_mixer_control_get_default_sink (dialog->priv->mixer_control);
         if (stream == NULL) {
@@ -217,6 +260,25 @@ update_output_settings (GvcMixerDialog *dialog)
                 num_settings++;
         }
 
+        ports = gvc_mixer_stream_get_ports (stream);
+        if (ports != NULL) {
+                GvcMixerStreamPort *port;
+                port = gvc_mixer_stream_get_port (stream);
+
+                dialog->priv->output_port_combo = gvc_combo_box_new (_("Connector:"));
+                gvc_combo_box_set_ports (GVC_COMBO_BOX (dialog->priv->output_port_combo),
+                                         ports);
+                gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->output_port_combo), port->port);
+                g_object_set_data (G_OBJECT (dialog->priv->output_port_combo), "stream", stream);
+                g_signal_connect (G_OBJECT (dialog->priv->output_port_combo), "changed",
+                                  G_CALLBACK (port_selection_changed), dialog);
+
+                gtk_box_pack_start (GTK_BOX (dialog->priv->output_settings_box),
+                                    dialog->priv->output_port_combo,
+                                    FALSE, FALSE, 12);
+                gtk_widget_show (dialog->priv->output_port_combo);
+        }
+
         /* FIXME: We could make this into a "No settings" label instead */
         gtk_widget_set_sensitive (dialog->priv->output_balance_bar, gvc_channel_map_can_balance (map));
 }
@@ -555,7 +617,10 @@ on_bar_is_muted_notify (GObject        *object,
         if (stream != NULL) {
                 gvc_mixer_stream_change_is_muted (stream, is_muted);
         } else {
-                g_warning ("Unable to find stream for bar");
+                char *name;
+                g_object_get (object, "name", &name, NULL);
+                g_warning ("Unable to find stream for bar '%s'", name);
+                g_free (name);
         }
 }
 
@@ -570,6 +635,60 @@ lookup_bar_for_stream (GvcMixerDialog *dialog,
         return bar;
 }
 
+static GtkWidget *
+lookup_combo_box_for_stream (GvcMixerDialog *dialog,
+                             GvcMixerStream *stream)
+{
+        GvcMixerStream *combo_stream;
+        guint id;
+
+        id = gvc_mixer_stream_get_id (stream);
+
+        if (dialog->priv->output_port_combo != NULL) {
+                combo_stream = g_object_get_data (G_OBJECT (dialog->priv->output_port_combo),
+                                                  "stream");
+                if (combo_stream != NULL) {
+                        if (id == gvc_mixer_stream_get_id (combo_stream))
+                                return dialog->priv->output_port_combo;
+                }
+        }
+
+        if (dialog->priv->input_port_combo != NULL) {
+                combo_stream = g_object_get_data (G_OBJECT (dialog->priv->input_port_combo),
+                                                  "stream");
+                if (combo_stream != NULL) {
+                        if (id == gvc_mixer_stream_get_id (combo_stream))
+                                return dialog->priv->input_port_combo;
+                }
+        }
+
+        return NULL;
+}
+
+static void
+on_stream_port_notify (GObject        *object,
+                       GParamSpec     *pspec,
+                       GvcMixerDialog *dialog)
+{
+        GvcComboBox *combo_box;
+        char *port;
+
+        combo_box = GVC_COMBO_BOX (lookup_combo_box_for_stream (dialog, GVC_MIXER_STREAM (object)));
+        if (combo_box == NULL)
+                return;
+
+        g_signal_handlers_block_by_func (G_OBJECT (combo_box),
+                                         port_selection_changed,
+                                         dialog);
+
+        g_object_get (object, "port", &port, NULL);
+        gvc_combo_box_set_active (GVC_COMBO_BOX (combo_box), port);
+
+        g_signal_handlers_unblock_by_func (G_OBJECT (combo_box),
+                                         port_selection_changed,
+                                         dialog);
+}
+
 static void
 on_stream_volume_notify (GObject        *object,
                          GParamSpec     *pspec,
@@ -683,6 +802,9 @@ bar_set_stream (GvcMixerDialog *dialog,
 
         adj = GTK_ADJUSTMENT (gvc_channel_bar_get_adjustment (GVC_CHANNEL_BAR (bar)));
 
+        g_object_set_data (G_OBJECT (bar), "gvc-mixer-dialog-stream", stream);
+        g_object_set_data (G_OBJECT (adj), "gvc-mixer-dialog-stream", stream);
+
         if (stream != NULL) {
                 is_muted = gvc_mixer_stream_get_is_muted (stream);
                 gvc_channel_bar_set_is_muted (GVC_CHANNEL_BAR (bar), is_muted);
@@ -692,9 +814,6 @@ bar_set_stream (GvcMixerDialog *dialog,
                 gtk_adjustment_set_value (adj,
                                           gvc_mixer_stream_get_volume (stream));
         }
-
-        g_object_set_data (G_OBJECT (bar), "gvc-mixer-dialog-stream", stream);
-        g_object_set_data (G_OBJECT (adj), "gvc-mixer-dialog-stream", stream);
 }
 
 static void
@@ -722,6 +841,7 @@ add_stream (GvcMixerDialog *dialog,
 
                 update_output_settings (dialog);
         } else if (stream == gvc_mixer_control_get_default_source (dialog->priv->mixer_control)) {
+                const GList *ports;
                 bar = dialog->priv->input_bar;
                 is_default = TRUE;
 
@@ -734,6 +854,30 @@ add_stream (GvcMixerDialog *dialog,
                 gvc_channel_bar_set_is_amplified (GVC_CHANNEL_BAR (bar),
                                                   gvc_mixer_stream_get_can_decibel (stream));
 
+                if (dialog->priv->input_port_combo != NULL) {
+                        gtk_container_remove (GTK_CONTAINER (dialog->priv->input_settings_box),
+                                              dialog->priv->input_port_combo);
+                        dialog->priv->input_port_combo = NULL;
+                }
+                ports = gvc_mixer_stream_get_ports (stream);
+                if (ports != NULL) {
+                        GvcMixerStreamPort *port;
+                        port = gvc_mixer_stream_get_port (stream);
+
+                        dialog->priv->input_port_combo = gvc_combo_box_new (_("Connector:"));
+                        gvc_combo_box_set_ports (GVC_COMBO_BOX (dialog->priv->input_port_combo),
+                                                 ports);
+                        gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->input_port_combo), port->port);
+                        g_object_set_data (G_OBJECT (dialog->priv->input_port_combo), "stream", stream);
+                        g_signal_connect (G_OBJECT (dialog->priv->input_port_combo), "changed",
+                                          G_CALLBACK (port_selection_changed), dialog);
+
+                        gtk_box_pack_start (GTK_BOX (dialog->priv->input_settings_box),
+                                            dialog->priv->input_port_combo,
+                                            TRUE, TRUE, 12);
+                        gtk_widget_show (dialog->priv->input_port_combo);
+                }
+
                 create_monitor_stream_for_source (dialog, stream);
         } else if (stream == gvc_mixer_control_get_event_sink_input (dialog->priv->mixer_control)) {
                 bar = dialog->priv->effects_bar;
@@ -805,6 +949,10 @@ add_stream (GvcMixerDialog *dialog,
                           "notify::volume",
                           G_CALLBACK (on_stream_volume_notify),
                           dialog);
+        g_signal_connect (stream,
+                          "notify::port",
+                          G_CALLBACK (on_stream_port_notify),
+                          dialog);
 }
 
 static void
@@ -828,8 +976,9 @@ on_control_stream_added (GvcMixerControl *control,
 }
 
 static gboolean
-find_stream_by_id (GtkTreeModel *model,
+find_item_by_id (GtkTreeModel *model,
                    guint         id,
+                   guint         column,
                    GtkTreeIter  *iter)
 {
         gboolean found_item;
@@ -844,7 +993,7 @@ find_stream_by_id (GtkTreeModel *model,
                 guint t_id;
 
                 gtk_tree_model_get (model, iter,
-                                    ID_COLUMN, &t_id, -1);
+                                    column, &t_id, -1);
 
                 if (id == t_id) {
                         found_item = TRUE;
@@ -882,12 +1031,12 @@ remove_stream (GvcMixerDialog  *dialog,
 
         /* remove from any models */
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->output_treeview));
-        found = find_stream_by_id (GTK_TREE_MODEL (model), id, &iter);
+        found = find_item_by_id (GTK_TREE_MODEL (model), id, ID_COLUMN, &iter);
         if (found) {
                 gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
         }
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->input_treeview));
-        found = find_stream_by_id (GTK_TREE_MODEL (model), id, &iter);
+        found = find_item_by_id (GTK_TREE_MODEL (model), id, ID_COLUMN, &iter);
         if (found) {
                 gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
         }
@@ -901,6 +1050,87 @@ on_control_stream_removed (GvcMixerControl *control,
         remove_stream (dialog, id);
 }
 
+static void
+add_card (GvcMixerDialog *dialog,
+          GvcMixerCard   *card)
+{
+        GtkTreeModel        *model;
+        GtkTreeIter          iter;
+        GtkTreeSelection    *selection;
+        GvcMixerCardProfile *profile;
+        GIcon               *icon;
+        guint                index;
+
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->hw_treeview));
+        index = gvc_mixer_card_get_index (card);
+        if (find_item_by_id (GTK_TREE_MODEL (model), index, HW_ID_COLUMN, &iter) == FALSE)
+                gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+        profile = gvc_mixer_card_get_profile (card);
+        g_assert (profile != NULL);
+        icon = g_themed_icon_new_with_default_fallbacks (gvc_mixer_card_get_icon_name (card));
+        //FIXME we need the status (default for a profile?) here
+        gtk_list_store_set (GTK_LIST_STORE (model),
+                            &iter,
+                            HW_NAME_COLUMN, gvc_mixer_card_get_name (card),
+                            HW_ID_COLUMN, index,
+                            HW_ICON_COLUMN, icon,
+                            HW_PROFILE_COLUMN, profile->profile,
+                            HW_PROFILE_HUMAN_COLUMN, profile->human_profile,
+                            HW_STATUS_COLUMN, profile->status,
+                            HW_SENSITIVE_COLUMN, g_strcmp0 (profile->profile, "off") != 0,
+                            -1);
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->priv->hw_treeview));
+        if (gtk_tree_selection_get_selected (selection, NULL, NULL) == FALSE) {
+                gtk_tree_selection_select_iter (selection, &iter);
+        } else if (dialog->priv->hw_profile_combo != NULL) {
+                GvcMixerCard *selected;
+
+                /* Set the current profile if it changed for the selected card */
+                selected = g_object_get_data (G_OBJECT (dialog->priv->hw_profile_combo), "card");
+                if (gvc_mixer_card_get_index (selected) == gvc_mixer_card_get_index (card)) {
+                        gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->hw_profile_combo),
+                                                  profile->profile);
+                }
+        }
+}
+
+static void
+on_control_card_added (GvcMixerControl *control,
+                       guint            id,
+                       GvcMixerDialog  *dialog)
+{
+        GvcMixerCard *card;
+
+        card = gvc_mixer_control_lookup_card_id (control, id);
+        if (card != NULL) {
+                add_card (dialog, card);
+        }
+}
+
+static void
+remove_card (GvcMixerDialog  *dialog,
+             guint            id)
+{
+        gboolean      found;
+        GtkTreeIter   iter;
+        GtkTreeModel *model;
+
+        /* remove from any models */
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->hw_treeview));
+        found = find_item_by_id (GTK_TREE_MODEL (model), id, HW_ID_COLUMN, &iter);
+        if (found) {
+                gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+        }
+}
+static void
+on_control_card_removed (GvcMixerControl *control,
+                         guint            id,
+                         GvcMixerDialog  *dialog)
+{
+        remove_card (dialog, id);
+}
+
 static void
 _gtk_label_make_bold (GtkLabel *label)
 {
@@ -1014,7 +1244,7 @@ name_to_text (GtkTreeViewColumn *column,
         } else {
                 char *str;
 
-                str = g_strdup_printf ("%s\n\t<i>%s</i>",
+                str = g_strdup_printf ("%s\n<i>%s</i>",
                                        name, mapping);
                 g_object_set (cell, "markup", str, NULL);
                 g_free (str);
@@ -1024,7 +1254,6 @@ name_to_text (GtkTreeViewColumn *column,
         g_free (mapping);
 }
 
-
 static GtkWidget *
 create_stream_treeview (GvcMixerDialog *dialog,
                         GCallback       on_toggled)
@@ -1074,6 +1303,151 @@ create_stream_treeview (GvcMixerDialog *dialog,
         return treeview;
 }
 
+static void
+on_profile_changed (GvcComboBox *widget,
+                    const char  *profile,
+                    gpointer     user_data)
+{
+        GvcMixerCard        *card;
+
+        card = g_object_get_data (G_OBJECT (widget), "card");
+        if (card == NULL) {
+                g_warning ("Could not find card for combobox");
+                return;
+        }
+
+        g_debug ("Profile changed to %s for card %s", profile,
+                 gvc_mixer_card_get_name (card));
+
+        gvc_mixer_card_change_profile (card, profile);
+}
+
+static void
+on_card_selection_changed (GtkTreeSelection *selection,
+                           gpointer          user_data)
+{
+        GvcMixerDialog      *dialog = GVC_MIXER_DIALOG (user_data);
+        GtkTreeModel        *model;
+        GtkTreeIter          iter;
+        const GList         *profiles;
+        guint                id;
+        GvcMixerCard        *card;
+        GvcMixerCardProfile *current_profile;
+
+        g_debug ("Card selection changed");
+
+        if (dialog->priv->hw_profile_combo != NULL) {
+                gtk_container_remove (GTK_CONTAINER (dialog->priv->hw_settings_box),
+                                      dialog->priv->hw_profile_combo);
+                dialog->priv->hw_profile_combo = NULL;
+        }
+
+        if (gtk_tree_selection_get_selected (selection,
+                                             NULL,
+                                             &iter) == FALSE) {
+                return;
+        }
+
+        model = gtk_tree_view_get_model (GTK_TREE_VIEW (dialog->priv->hw_treeview));
+        gtk_tree_model_get (model, &iter,
+                            HW_ID_COLUMN, &id,
+                            -1);
+        card = gvc_mixer_control_lookup_card_id (dialog->priv->mixer_control, id);
+        if (card == NULL) {
+                g_warning ("Unable to find card for id: %u", id);
+                return;
+        }
+
+        current_profile = gvc_mixer_card_get_profile (card);
+        profiles = gvc_mixer_card_get_profiles (card);
+        dialog->priv->hw_profile_combo = gvc_combo_box_new (_("Profile:"));
+        gvc_combo_box_set_profiles (GVC_COMBO_BOX (dialog->priv->hw_profile_combo), profiles);
+        gvc_combo_box_set_active (GVC_COMBO_BOX (dialog->priv->hw_profile_combo), current_profile->profile);
+
+        gtk_box_pack_start (GTK_BOX (dialog->priv->hw_settings_box),
+                            dialog->priv->hw_profile_combo,
+                            FALSE, FALSE, 12);
+        gtk_widget_show (dialog->priv->hw_profile_combo);
+
+        g_object_set_data (G_OBJECT (dialog->priv->hw_profile_combo), "card", card);
+        g_signal_connect (G_OBJECT (dialog->priv->hw_profile_combo), "changed",
+                          G_CALLBACK (on_profile_changed), dialog);
+}
+
+static void
+card_to_text (GtkTreeViewColumn *column,
+              GtkCellRenderer *cell,
+              GtkTreeModel *model,
+              GtkTreeIter *iter,
+              gpointer user_data)
+{
+        char *name, *status, *profile, *str;
+        gboolean sensitive;
+
+        gtk_tree_model_get(model, iter,
+                           HW_NAME_COLUMN, &name,
+                           HW_STATUS_COLUMN, &status,
+                           HW_PROFILE_HUMAN_COLUMN, &profile,
+                           HW_SENSITIVE_COLUMN, &sensitive,
+                           -1);
+
+        str = g_strdup_printf ("%s\n<i>%s</i>\n<i>%s</i>",
+                               name, status, profile);
+        g_object_set (cell,
+                      "markup", str,
+                      "sensitive", sensitive,
+                      NULL);
+        g_free (str);
+
+        g_free (name);
+        g_free (status);
+        g_free (profile);
+}
+
+static GtkWidget *
+create_cards_treeview (GvcMixerDialog *dialog,
+                       GCallback       on_changed)
+{
+        GtkWidget         *treeview;
+        GtkListStore      *store;
+        GtkCellRenderer   *renderer;
+        GtkTreeViewColumn *column;
+        GtkTreeSelection  *selection;
+
+        treeview = gtk_tree_view_new ();
+        gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+        g_signal_connect (G_OBJECT (selection), "changed",
+                          on_changed, dialog);
+
+        store = gtk_list_store_new (HW_NUM_COLUMNS,
+                                    G_TYPE_UINT,
+                                    G_TYPE_ICON,
+                                    G_TYPE_STRING,
+                                    G_TYPE_STRING,
+                                    G_TYPE_STRING,
+                                    G_TYPE_STRING,
+                                    G_TYPE_BOOLEAN);
+        gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
+                                 GTK_TREE_MODEL (store));
+
+        renderer = gtk_cell_renderer_pixbuf_new ();
+        g_object_set (G_OBJECT (renderer), "stock-size", GTK_ICON_SIZE_DIALOG, NULL);
+        column = gtk_tree_view_column_new_with_attributes (NULL,
+                                                           renderer,
+                                                           "gicon", HW_ICON_COLUMN,
+                                                           "sensitive", HW_SENSITIVE_COLUMN,
+                                                           NULL);
+        gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
+        gtk_tree_view_insert_column_with_data_func (GTK_TREE_VIEW (treeview), -1,
+                                                    _("Name"), gtk_cell_renderer_text_new (),
+                                                    card_to_text, NULL, NULL);
+
+        return treeview;
+}
+
 static const guint tab_accel_keys[] = {
         GDK_1, GDK_2, GDK_3, GDK_4
 };
@@ -1114,8 +1488,10 @@ gvc_mixer_dialog_constructor (GType                  type,
         GtkWidget        *sbox;
         GtkWidget        *ebox;
         GSList           *streams;
+        GSList           *cards;
         GSList           *l;
         GvcMixerStream   *stream;
+        GvcMixerCard     *card;
         GtkTreeSelection *selection;
         GtkAccelGroup    *accel_group;
         GClosure         *closure;
@@ -1183,6 +1559,46 @@ gvc_mixer_dialog_constructor (GType                  type,
                             self->priv->sound_theme_chooser,
                             TRUE, TRUE, 6);
 
+        /* Hardware page */
+        self->priv->hw_box = gtk_vbox_new (FALSE, 12);
+        gtk_container_set_border_width (GTK_CONTAINER (self->priv->hw_box), 12);
+        label = gtk_label_new (_("Hardware"));
+        gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
+                                  self->priv->hw_box,
+                                  label);
+
+        box = gtk_frame_new (_("Choose a device to configure"));
+        label = gtk_frame_get_label_widget (GTK_FRAME (box));
+        _gtk_label_make_bold (GTK_LABEL (label));
+        gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
+        gtk_box_pack_start (GTK_BOX (self->priv->hw_box), box, TRUE, TRUE, 0);
+
+        alignment = gtk_alignment_new (0, 0, 1, 1);
+        gtk_container_add (GTK_CONTAINER (box), alignment);
+        gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 6, 0, 0, 0);
+
+        self->priv->hw_treeview = create_cards_treeview (self,
+                                                         G_CALLBACK (on_card_selection_changed));
+        box = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (box),
+                                        GTK_POLICY_NEVER,
+                                        GTK_POLICY_AUTOMATIC);
+        gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (box),
+                                             GTK_SHADOW_IN);
+        gtk_container_add (GTK_CONTAINER (box), self->priv->hw_treeview);
+        gtk_container_add (GTK_CONTAINER (alignment), box);
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (self->priv->hw_treeview));
+        gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+
+        box = gtk_frame_new (_("Settings for the selected device:"));
+        label = gtk_frame_get_label_widget (GTK_FRAME (box));
+        _gtk_label_make_bold (GTK_LABEL (label));
+        gtk_frame_set_shadow_type (GTK_FRAME (box), GTK_SHADOW_NONE);
+        gtk_box_pack_start (GTK_BOX (self->priv->hw_box), box, FALSE, TRUE, 12);
+        self->priv->hw_settings_box = gtk_vbox_new (FALSE, 12);
+        gtk_container_add (GTK_CONTAINER (box), self->priv->hw_settings_box);
+
         /* Input page */
         self->priv->input_box = gtk_vbox_new (FALSE, 12);
         gtk_container_set_border_width (GTK_CONTAINER (self->priv->input_box), 12);
@@ -1235,6 +1651,11 @@ gvc_mixer_dialog_constructor (GType                  type,
                             FALSE, FALSE, 0);
         gtk_size_group_add_widget (self->priv->size_group, ebox);
 
+        self->priv->input_settings_box = gtk_hbox_new (FALSE, 6);
+        gtk_box_pack_start (GTK_BOX (self->priv->input_box),
+                            self->priv->input_settings_box,
+                            FALSE, FALSE, 0);
+
         box = gtk_frame_new (_("Choose a device for sound input"));
         label = gtk_frame_get_label_widget (GTK_FRAME (box));
         _gtk_label_make_bold (GTK_LABEL (label));
@@ -1319,6 +1740,14 @@ gvc_mixer_dialog_constructor (GType                  type,
                           "stream-removed",
                           G_CALLBACK (on_control_stream_removed),
                           self);
+        g_signal_connect (self->priv->mixer_control,
+                          "card-added",
+                          G_CALLBACK (on_control_card_added),
+                          self);
+        g_signal_connect (self->priv->mixer_control,
+                          "card-removed",
+                          G_CALLBACK (on_control_card_removed),
+                          self);
 
         gtk_widget_show_all (GTK_WIDGET (self));
 
@@ -1329,6 +1758,13 @@ gvc_mixer_dialog_constructor (GType                  type,
         }
         g_slist_free (streams);
 
+        cards = gvc_mixer_control_get_cards (self->priv->mixer_control);
+        for (l = cards; l != NULL; l = l->next) {
+                card = l->data;
+                add_card (self, card);
+        }
+        g_slist_free (cards);
+
         return object;
 }
 
@@ -1344,6 +1780,12 @@ gvc_mixer_dialog_dispose (GObject *object)
                 g_signal_handlers_disconnect_by_func (dialog->priv->mixer_control,
                                                       on_control_stream_removed,
                                                       dialog);
+                g_signal_handlers_disconnect_by_func (dialog->priv->mixer_control,
+                                                      on_control_card_added,
+                                                      dialog);
+                g_signal_handlers_disconnect_by_func (dialog->priv->mixer_control,
+                                                      on_control_card_removed,
+                                                      dialog);
 
                 g_object_unref (dialog->priv->mixer_control);
                 dialog->priv->mixer_control = NULL;
